package Oryx::DBM::Association;
use base qw(Oryx::Association);
1;
__END__

=head1 NAME

Oryx::DBM::Association - Abstract base for DBM association implementations

=head1 SYNOPSIS

See L<Oryx::Association>.

=head1 DESCRIPTION

This is an abstract base class for the DBM implementations of associations.

=head1 SEE ALSO

L<Oryx>, L<Oryx::DBM>, L<Oryx::Association>, L<Oryx::DBM::Association::Array>, L<Oryx::DBM::Association::Hash>, L<Oryx::DBM::Association::Reference>

=head1 AUTHOR

Richard Hundt E<lt>richard NO SPAM AT protea-systems.comE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (c) 2005 Richard Hundt.

This library is free software and may be used under the same terms as Perl itself.

=cut
